package org.processmining.baseminers.algorithms;

import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.transitionsystem.TransitionSystem;
import org.processmining.plugins.transitionsystem.miner.TSMinerInput;

public abstract class AbstractMinerAlgorithm {

	private int threshold;
	
	public AbstractMinerAlgorithm() {
		threshold = 0; // Keep all transitions.
	}
	
	public AbstractMinerAlgorithm(int threshold) {
		this.threshold = threshold;
	}
	
	public Petrinet apply(PluginContext context, XLog log, XEventClassifier classifier) {

		// Create the required input for the transition system miner.
		Set<XEventClassifier> classifiers = new HashSet<XEventClassifier>();
		classifiers.add(classifier);
		TSMinerInput input = getInput(context, log, classifiers, classifier);

		// Run the transition system miner.
		TransitionSystem transitionSystem;
		try {
			transitionSystem = context.tryToFindOrConstructFirstNamedObject(TransitionSystem.class, "Mine Transition System", null, null,
					log, classifiers.toArray(new XEventClassifier[0]), classifier, input);
		} catch (ConnectionCannotBeObtained e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}

		// Filter the transition system.
		transitionSystem = filter(context, log, classifier, transitionSystem, threshold);
		
		// Convert the transition system to a Petri net.
		Petrinet petrinet;
		try {
			petrinet = context.tryToFindOrConstructFirstNamedObject(Petrinet.class, "Convert Transition System to (State Machine) Petri Net", null, null, transitionSystem);
		} catch (ConnectionCannotBeObtained e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		
		// Return the Petri net.
		return petrinet;
	}

	public abstract TSMinerInput getInput(PluginContext context, XLog log, Set<XEventClassifier> classifiers, XEventClassifier classifier);
	
	public TransitionSystem filter(PluginContext context, XLog log, XEventClassifier classifier, TransitionSystem transitionSystem, int threshold) {
		// By default: return provided transition system.
		return transitionSystem;
	}
}
