package org.processmining.baseminers.algorithms;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.connections.transitionsystem.TransitionSystemConnection;
import org.processmining.models.graphbased.directed.DirectedGraphElementWeights;
import org.processmining.models.graphbased.directed.transitionsystem.TransitionSystem;
import org.processmining.plugins.transitionsystem.miner.TSMinerInput;

public class DirectlyFollowsFilteredMinerAlgorithm extends DirectlyFollowsMinerAlgorithm {

	public DirectlyFollowsFilteredMinerAlgorithm(int threshold) {
		super(threshold);
	}

	public TransitionSystem filter(PluginContext context, XLog log, XEventClassifier classifier, TransitionSystem transitionSystem, int threshold) {
		TransitionSystemConnection conn;
		try {
			conn = context.getConnectionManager().getFirstConnection(TransitionSystemConnection.class, context,
					transitionSystem);
		} catch (ConnectionCannotBeObtained e) {
			return transitionSystem;
		}
		if (conn != null) {
			// Map every possible transition (and state) to its weight.
			// Use String representation for sake of ease.
			DirectedGraphElementWeights weights = conn.getObjectWithRole(TransitionSystemConnection.WEIGHTS);
			Map<String, Integer> w = new HashMap<String, Integer>();
			for (Object object : weights.keySet()) {
				w.put(object.toString(), weights.get(object));
			}
			// Compute for every trace the minimal weight over all transitions in the trace.
			Map<XTrace, Integer> minWeights = new HashMap<XTrace, Integer>();
			for (XTrace trace : log) {
				String targetState = "";
				String sourceState = "";
				int weight = 0;
				int minWeight = Integer.MAX_VALUE;
				for (XEvent event : trace) {
					targetState = classifier.getClassIdentity(event);
					// Put the transition into the correct String representation.
					weight = w.get("[[" + sourceState + "], [" + targetState + "], " + targetState + "]");
					if (weight < minWeight) 
						minWeight = weight;
					sourceState = targetState;
				}
				minWeights.put(trace, minWeight);
			}
			
			// Create a filtered log that includes at least (100-threshold)% of the 'most-heavy' traces
			Set<Integer> uniqueWeights = new HashSet<Integer>(minWeights.values());
			List<Integer> sortedWeights = new ArrayList<Integer>(uniqueWeights);
			Collections.sort(sortedWeights);
			int minNofTraces = (log.size() * (100 - threshold)) / 100;
			XLog filteredLog = XFactoryRegistry.instance().currentDefault().createLog();
			int index = sortedWeights.size();
			while (filteredLog.isEmpty() || filteredLog.size() < minNofTraces) {
				int weight = sortedWeights.get(--index);
				filteredLog.clear();
				for (XTrace trace : log) {
					if (minWeights.get(trace) >= weight) {
						filteredLog.add(trace);
					}
				}
			}
			
			// Mine the same transition system but now for the filtered log, and return that one.
			Set<XEventClassifier> classifiers = new HashSet<XEventClassifier>();
			classifiers.add(classifier);
			TSMinerInput input = getInput(context, filteredLog, classifiers, classifier);
			try {
				return context.tryToFindOrConstructFirstNamedObject(TransitionSystem.class, "Mine Transition System", null, null,
						filteredLog, classifiers.toArray(new XEventClassifier[0]), classifier, input);
			} catch (ConnectionCannotBeObtained e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				return transitionSystem;
			}
		}
		return transitionSystem;
	}
}
