package org.processmining.baseminers.plugins;

import org.processmining.baseminers.algorithms.ConvertTS2PNAlgorithm;
import org.processmining.baseminers.parameters.ConvertTS2PNParameters;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.transitionsystem.TransitionSystem;

@Plugin(name = "Convert Transition System to (State Machine) Petri Net", level = PluginLevel.Regular, parameterLabels = {
		"Transition System", "Parameters" }, returnLabels = {
				"(State Machine) Petri Net" }, returnTypes = { Petrinet.class }, userAccessible = true, help = "")
public class ConvertTS2PNPlugin extends ConvertTS2PNAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Convert Transition System to (State Machine) Petri Net, default parameters", requiredParameterLabels = {
			0 })
	public Petrinet run(PluginContext context, TransitionSystem transitionSystem) {
		return apply(context, transitionSystem, new ConvertTS2PNParameters());
	}

	@PluginVariant(variantLabel = "Convert Transition System to (State Machine) Petri Net, provided parameters", requiredParameterLabels = {
			0, 1 })
	public Petrinet run(PluginContext context, TransitionSystem transitionSystem, ConvertTS2PNParameters parameters) {
		return apply(context, transitionSystem, parameters);
	}

}
